/*****************************************************************
 * skin_parsing.c
 *
 * Copyright(C)2003, Julien Delacroix
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GPL.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "skin_data.h"
#include "skin_parsing.h"

void start_hndl(void *data, const char *el, const char **attr) {
  struct context_info *p = (struct context_info *) data;
  
#ifdef DEBUG
  printf("start %s\n",el);
#endif
  
  switch(p->depth)
    {
    case 1: /* starting new column */
      p->current_column = (struct column *)malloc(sizeof(struct column));
      break;
      
    case 2:
      p->waiting_for_text = TRUE;
      if (!strcmp(el,"content"))
	p->last_tag = CONTENT; 
      else if (!strcmp(el,"title"))
	p->last_tag = TITLE;
      else if (!strcmp(el,"length"))
	p->last_tag = LENGTH;
      else{
	p->waiting_for_text = TRUE;
	p->waiting_for_text = FALSE;
	if( !strcmp(el,"title_format") || !strcmp(el,"default_format")){
	  p->current_cell = (struct cell *)malloc(sizeof(struct cell));
	}
      }
      break;
      
    case 3:
      p->waiting_for_text = TRUE;
      if (!strcmp(el,"fg"))
	p->last_tag = FG; 
      else if (!strcmp(el,"bg"))
	p->last_tag = BG;
      else if (!strcmp(el,"style"))
	p->last_tag = STYLE;
      else if (!strcmp(el,"align"))
	p->last_tag = ALIGN;
      break;
    }
  p->depth++;
}  /* End of start_hndl */


void end_hndl(void *data, const char *el) {
  struct context_info *p = (struct context_info *) data;
  
#ifdef DEBUG
  printf("end %s\n\n",el);
#endif
  
  p->depth--;  
  switch(p->depth)
    {
    case 1:
      SKIN_LIST_APPEND(p->skin, p->current_column);
      break;
      
    case 2:
      if (!strcmp(el,"title_format"))
	p->current_column->title_format = p->current_cell;
      
      else if (!strcmp(el,"default_format"))
	p->current_column->default_format = p->current_cell;
      break;
    }
}  /* End of end_hndl */

void char_hndl(void *data, const char *txt, int txtlen){
  struct context_info *p = (struct context_info *)data;
  char value[40];
  if (p->waiting_for_text==TRUE){
    strncpy(value,txt,txtlen);
    value[txtlen]='\0';

#ifdef DEBUG
    printf("text %s\n",value);
#endif

    switch(p->last_tag){
    case TITLE: 
      p->current_column->title = (char *) malloc(sizeof(char)*40);
      strcpy(p->current_column->title, value);
      break;
      
    case CONTENT:
      p->current_column->content = get_content(value,p->parser);
      break;
   
    case LENGTH:
      p->current_column->length = get_length(value,p->parser);
      break;
     
    case FG:
      p->current_cell->fg_color = get_color(value,p->parser);
      break;
      
    case BG:
      p->current_cell->bg_color = get_color(value,p->parser);
      break;

    case STYLE:
      p->current_cell->style = get_style(value,p->parser);
      break;

    case ALIGN:
      p->current_cell->alignment = get_alignment(value,p->parser);
      break;
    }
    p->waiting_for_text = FALSE;
  }
}  /* End char_hndl */


struct context_info *initialise_context_info(XML_Parser p){
  struct context_info *data = (struct context_info *)malloc(sizeof(struct context_info));
  data->depth = 0;
  data->last_tag = NONE; 
  data->waiting_for_text = FALSE;

  data->current_column = NULL;
  data->current_cell = NULL;	
  data->skin = NULL ;
  data->parser = p;
  return(data);
}

struct column *skin_parse(char *skin_file) {
  char Buff[512]; /* should be enough for a line... */
  struct context_info *data;
  FILE *fp;
  XML_Parser p;

  p = XML_ParserCreate(NULL);
  if (! p) {
    fprintf(stderr, "Couldn't allocate memory for parser\n");
    exit(-1);
  }
  data = initialise_context_info(p);
  XML_SetElementHandler(p, start_hndl, end_hndl);
  XML_SetCharacterDataHandler(p, char_hndl);
  XML_SetUserData(p, (void *)data);

  fp=fopen(skin_file,"r");
  if (fp==-1){
    fprintf(stderr,"Cant open file %s\n",skin_file);
  exit(-1);
  }/* End of if */
			
  for (;;) {
    int done;
    int len;
    fgets(Buff, sizeof(Buff), fp);
    len = strlen(Buff);
    if (ferror(fp)) {
      fprintf(stderr, "Read error\n");
      exit(-1);
    }
    done = feof(fp);
    if (! XML_Parse(p, Buff, len, done)) {
      fprintf(stderr, "Parse error at line %d:\n%s\n",
	      XML_GetCurrentLineNumber(p),
	      XML_ErrorString(XML_GetErrorCode(p)));
      exit(-1);
    }
    if (done)
      break;
  }
  XML_ParserFree(p);
  return data->skin;	
}  /* End of skin_parse */

