#include "trace.h"

#include <string.h>
#include <locale.h>
#include <time.h>
#include <pwd.h>

char * getName (const char * login, char * name, const size_t n)
{
	struct passwd * infos ;

	if (! login || ! name)
		return NULL ;
	infos = getpwnam (login) ;
	if (! infos || ! infos->pw_gecos)
		return NULL ;
	return strncpy (name, strtok (infos->pw_gecos, ","), n) ;
}

int getMonthI (const char * month, const char * locale)
{
	int status = -1 ;
	int i ;
	char * localeSave = NULL ;
	struct tm date ;
	char month2 [4] ;

	if (! month || strlen (month) != 3)
		goto finally ;

	localeSave = setlocale (LC_TIME, NULL) ;
	setlocale (LC_TIME, locale) ;

	for (i = 0 ; i < 11 ; i ++)
	{
		date.tm_mon = i ;
		if (strftime (month2, 4, "%b", & date) < 3)
			goto finally ;
		if (! strcmp (month, month2))
		{
			status = i ;
			goto finally ;
		}
	}

finally :
	if (localeSave)
		setlocale (LC_TIME, localeSave) ;

	return status ;
}

int trace_getUnixFrom (const char * login, trace_from * unixFrom)
{
	int status = 0 ;
	FILE * out = NULL ;
	char command [150] ;
	char month [4] ;
	time_t actualTime ;
	struct tm * actualStruct ;

	if (! login || ! unixFrom)
	{
		status = -1 ;
		goto finally ;
	}

	if (snprintf (command, 150, "last %s | grep 'wtmp begins'", login) < 0)
	{
		status = -1 ;
		goto finally ;
	}

	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}

	if (fscanf (out, "%*16c%s%d%d:%d", month, & unixFrom->day, & unixFrom->hour, & unixFrom->min) != 4)
	{
		status = -1 ;
		goto finally ;
	}

	unixFrom->day -- ;
	unixFrom->month = getMonthI (month, "") ;
	if (unixFrom->month < 0)
	{
		status = -1 ;
		goto finally ;
	}
	if (time (& actualTime) < 0)
	{
		status = -1 ;
		goto finally ;
	}
	actualStruct = localtime (& actualTime) ;
	if (! actualStruct)
	{
		status = -1 ;
		goto finally ;
	}
	unixFrom->year = actualStruct->tm_year ;
	if (actualStruct->tm_mon < unixFrom->month)
		unixFrom->year -- ;
	unixFrom->year += 1900 ;

finally :
	if (out)
		pclose (out) ;

	return status ;
}

int trace_getWinFrom (trace_from * winFrom)
{
	int status = 0 ;
	FILE * out = NULL ;
	char command [150] ;

	if (! winFrom)
	{
		status = -1 ;
		goto finally ;
	}

	if (snprintf (command, 150, "head -n 1 /usr/local/samba/var/log.smb.old") < 0)
	{
		status = -1 ;
		goto finally ;
	}

	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}

	if (fscanf (out, "[%d/%d/%d %d:%d", & winFrom->year, & winFrom->month, & winFrom->day, & winFrom->hour, & winFrom->min) != 5)
	{
		status = -1 ;
		goto finally ;
	}

	winFrom->month -- ;
	winFrom->day -- ;

finally :
	if (out)
		pclose (out) ;

	return status ;
}

int trace_getUnixLogs (const char * login, trace_logs * unixLogs)
{
	int status = 0 ;
	FILE * out = NULL ;
	char command [150] ;
	char line [150] ;
	trace_log * log ;
	char month [4] ;
	time_t actualTime ;
	struct tm * actualStruct ;

	if (! login || ! unixLogs)
	{
		status = -1 ;
		goto finally ;
	}

	if (snprintf (command, 150, "last %s | grep %s", login, login) < 0)
	{
		status = -1 ;
		goto finally ;
	}

	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}

	while (fgets (line, 150, out))
	{
		log = unixLogs->logsArray + unixLogs->logsNb ;

		log->os = 'u' ;

		if (! strncmp (line + 59, "still logged in", 15))
		{
			log->endHour = TIME_NOT_YET ;
			log->endMin = TIME_NOT_YET ;
		}
		else
		{
			log->endHour = TIME_UNKNOWN ;
			log->endMin = TIME_UNKNOWN ;
		}
		if (sscanf (line + 44, "%s %d %d:%d - %d:%d", month, & log->day, & log->beginHour, & log->beginMin, & log->endHour, & log->endMin) < 4)
		{
			status = -1 ;
			goto finally ;
		}

		log->day -- ;
		log->month = getMonthI (month, "") ;
		if (log->month < 0)
		{
			status = -1 ;
			goto finally ;
		}
		if (time (& actualTime) < 0)
		{
			status = -1 ;
			goto finally ;
		}
		actualStruct = localtime (& actualTime) ;
		if (! actualStruct)
		{
			status = -1 ;
			goto finally ;
		}
		log->year = actualStruct->tm_year ;
		if (actualStruct->tm_mon < log->month)
			log->year -- ;
		log->year += 1900 ;

		if (! strncpy (log->host, line + 23, 16))
		{
			status = -1 ;
			goto finally ;
		}
		strtok (log->host, " ") ;

		unixLogs->logsNb ++ ;
	}

finally :
	if (out)
		pclose (out) ;

	return status ;
}

int trace_getWinLogs (const char * login, trace_logs * winLogs)
{
	int status = 0 ;
	FILE * out = NULL ;
	char command [150] ;
	char line1 [150] ;
	char line2 [150] ;
	trace_log * log ;
	char month [4] ;

	if (! login || ! winLogs)
	{
		status = -1 ;
		goto finally ;
	}

	if (snprintf (command, 150, "cat /usr/local/samba/var/log.smb.old /usr/local/samba/var/log.smb | grep -B 1 %s | grep -A 1 service.c", login) < 0)
	{
		status = -1 ;
		goto finally ;
	}

	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}
	while (fgets (line1, 150, out))
	{
		if (* line1 == '-')
			fgets (line1, 150, out) ;
		fgets (line2, 150, out) ;
		if (! strstr (line1, "close_cnum"))
			continue ;

		log = winLogs->logsArray + winLogs->logsNb ;

		log->os = 'w' ;

		if (sscanf (line1, "[%d/%d/%d %d:%d", & log->year, & log->month, & log->day, & log->endHour, & log->endMin) < 5)
		{
			status = -1 ;
			goto finally ;
		}
		log->month -- ;
		log->day -- ;
		log->beginHour = TIME_UNKNOWN ;
		log->beginMin = TIME_UNKNOWN ;

		if (! strncpy (log->host, line2 + 2, 30))
		{
			status = -1 ;
			goto finally ;
		}
		strtok (log->host, " ") ;

		winLogs->logsNb ++ ;
	}
	pclose (out) ;
	out = NULL ;

	if (snprintf (command, 150, "/usr/local/samba/bin/smbstatus.old -bu %s | cut -c9- | grep %s", login, login) < 0)
	{
		status = -1 ;
		goto finally ;
	}

	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}
	while (fgets (line1, 150, out))
	{
		log = winLogs->logsArray + winLogs->logsNb ;

		log->os = 'w' ;

		if (sscanf (line1 + 44, "%s %d %d:%d:%*d %d", month, & log->day, & log->beginHour, & log->beginMin, & log->year) != 5)
		{
			status = -1 ;
			goto finally ;
		}
		log->month = getMonthI (month, "C") ;
		if (log->month < 0)
		{
			status = -1 ;
			goto finally ;
		}
		log->day -- ;
		log->endHour = TIME_NOT_YET ;
		log->endMin = TIME_NOT_YET ;

		if (! strncpy (log->host, line1 + 10, 30))
		{
			status = -1 ;
			goto finally ;
		}
		strtok (log->host, " ") ;

		winLogs->logsNb ++ ;
	}

finally :
	if (out)
		pclose (out) ;

	return status ;
}

int trace_getLogsNb (const char * login)
{
	int status = 0 ;
	FILE * out = NULL ;
	char command [200] ;
	int logsNb ;

	if (! login)
	{
		status = -1 ;
		goto finally ;
	}
	
	if (snprintf (command, 200, "((cat /usr/local/samba/var/log.smb.old /usr/local/samba/var/log.smb | grep closed) ; /usr/local/samba/bin/smbstatus.old -bu %s ; last %s) | grep %s | wc -l", login, login, login) < 0)
	{
		status = -1 ;
		goto finally ;
	}
	
	out = popen (command, "r") ;
	if (! out)
	{
		status = -1 ;
		goto finally ;
	}
	
	if (fscanf (out, "%d", & logsNb) != 1)
	{
		status = -1 ;
		goto finally ;
	}
	
finally :
	if (out)
		pclose (out) ;

	return (status ? status : logsNb) ;
}
