#include "trace.h"

#include <string.h>
#include <time.h>
#include "../kelsal/kelsal.h"

int trace_printMonth (const int month)
{
	int status = 0 ;
	struct tm dateInt ;
	char monthStr [4] ;

	dateInt.tm_mon = month ;
	status |= strftime (monthStr, 4, "%b", & dateInt) != 3 ;
	status |= printf ("%s", monthStr) < 3 ;

	return status ;
}

int trace_printDay (const int year, const int month, const int day)
{
	int status = 0 ;
	struct tm dateInt ;
	time_t dateSec ;
	char dayStr [4] ;

	dateInt.tm_year = year - 1900 ;
	dateInt.tm_mon = month ;
	dateInt.tm_mday = day + 1 ;
	dateInt.tm_isdst = 0 ;
	dateInt.tm_hour = 0 ;
	dateInt.tm_min = 0 ;
	dateInt.tm_sec = 0 ;
	dateSec = mktime (& dateInt) ;
	if (dateSec < 0)
		return -1 ;
	status |= strftime (dayStr, 4, "%a", localtime (& dateSec)) != 3 ;
	status |= printf ("%s", dayStr) < 0 ;

	return status ;
}

int trace_printTime (const int hour, const int min)
{
	int status = 0 ;
	struct tm timeInt ;
	char timeChar [6] ;

	if (hour == TIME_UNKNOWN)
		return printf (" ??? ") != 5 ;

	timeInt.tm_hour = hour ;
	timeInt.tm_min = min ;
	status |= strftime (timeChar, 6, "%H:%M", & timeInt) != 5 ;
	status |= printf ("%s", timeChar) != 5 ;

	return status ;
}

int trace_printHeader (const char * name, const trace_from * from)
{
	int status = 0 ;
	int i ;

	if (! name || ! from)
		return -1 ;
	status |= printf ("\x1B[7m") < 0 ;
	status |= printf ("%s", name) < 0 ;
	for (i = strlen (name) ; i < 30 ; i ++)
		status |= printf (" ") < 0 ;
	status |= printf ("depuis ") < 0 ;
	status |= trace_printDay (from->year, from->month, from->day) ;
	status |= printf (" ") < 0 ;
	status |= printf ("%d", from->day + 1) < 0 ;
	status |= printf (" ") < 0 ;
	status |= trace_printMonth (from->month) ;
	status |= printf (" ") < 0 ;
	status |= trace_printTime (from->hour, from->min) ;
	status |= printf ("\x1B[0m\n") < 0 ;

	return status ;
}

int trace_printLog (const trace_log * log)
{
	int status = 0 ;
	char * host, * place ;

	status |= trace_printMonth (log->month) ;
	status |= printf (" ") < 0 ;
	status |= printf ("%2d", log->day + 1) < 0 ;
	status |= printf (" ") < 0 ;
	status |= trace_printDay (log->year, log->month, log->day) ;
	status |= printf ("  ") != 2 ;
	status |= trace_printTime (log->beginHour, log->beginMin) ;
	if (log->endHour == TIME_NOT_YET)
		status |= printf (" \x1B[4mconnect\x1B[0m ") < 0 ;
	else
	{
		status |= printf (" - ") != 3 ;
		status |= trace_printTime (log->endHour, log->endMin) ;
		status |= printf ("  ") != 2 ;
	}
	host = (char *) log->host ;
	place = kelsal (host) ;
	status |= printf ("%c  %s\n", log->os, place ? place : host) < 3 ;

	return status ;
}

int trace_printLogsAfter (const trace_logs * logs, const trace_from * from)
{
	int status = 0 ;
	int logsIndex ;

	if (! logs || ! from)
		return -1 ;

	if (! logs->logsNb)
		return printf ("aucune connexion\n") < 0 ;
	
	for (logsIndex = 0 ; logsIndex < logs->logsNb ; logsIndex ++)
	{
		if (trace_compareLogFrom (logs->logsArray + logsIndex, from) >= 0)
			status |= trace_printLog (logs->logsArray + logsIndex) ;
	}

	return status ;
}
